@@ -39,22 +39,19 @@ The need to explicitly C<use Gimp::Util> will go away in the future.
 =cut
 
 package      Gimp::Util;
-use Exporter 'import';
-use vars qw(@EXPORT $VERSION);
-use strict;
+require      Exporter;
+@ISA       = qw(Exporter);
 @EXPORT    = qw(
-                layer_create
-                text_draw
+                layer_create 
+                text_draw 
                 image_create_text
                 layer_add_layer_as_mask
                );
 #@EXPORT_OK = qw();
 
-import Gimp;
-# manual "import" to shut perl -cw up
-sub __ ($);
+use Gimp;
 
-$VERSION = 2.3000_01;
+$VERSION=2.3;
 
 ##############################################################################
 =pod
@@ -104,15 +101,15 @@ sub layer_create {
   my $tcol; # scratch color
 
   # create a colored layer
-  $layer = Gimp::Layer->new ($image,Gimp->image_width($image),
+  $layer = Gimp->layer_new ($image,Gimp->image_width($image),
                            Gimp->image_height($image),
-                           &RGB_IMAGE,$name,100,&NORMAL_MODE);
+                           RGB_IMAGE,$name,100,NORMAL_MODE);
   $tcol = Gimp->palette_get_background ();
   Gimp->palette_set_background ($color);
-  Gimp->drawable_fill ($layer,&BACKGROUND_FILL);
+  Gimp->drawable_fill ($layer,BACKGROUND_FILL);
   Gimp->image_add_layer($image, $layer, $pos);
   Gimp->palette_set_background ($tcol); # reset
-  $layer;
+  $layer;  
   }
 
 ##############################################################################
@@ -139,20 +136,31 @@ sub text_draw {
   Gimp->palette_set_foreground ($fgcolor);
   # Create a layer for the text.
   $text_layer = Gimp->text($image,-1,0,0,$text,10,1,$size,
-			    &PIXELS,"*",$font,"*","*","*","*");
-
+			    PIXELS,"*",$font,"*","*","*","*"); 
+    
   # Do the fun stuff with the text.
-  Gimp->layer_set_preserve_trans($text_layer, &FALSE);
+  Gimp->layer_set_preserve_trans($text_layer, FALSE);
 
+  if ($resize == 0)
+    {
+    # Now figure out the size of $image
+    $width = Gimp->image_width($text_layer);
+    $height = Gimp->image_height($text_layer);
+    # and cut text layer
+    }
+  else
+    {
+    }
+					  
   # add text to image
-  Gimp->image_add_layer($image, $text_layer, 0);
+  Gimp->image_add_layer($image, $text_layer, $pos);
   # merge white and text
   Gimp->image_merge_visible_layers ($image,1);
-  # cleanup the left over layer (!)
+  # cleanup the left over layer (!) 
   Gimp->layer_delete($text_layer);
   $layer;
 }
-
+    
 ##############################################################################
 =pod
 
@@ -177,16 +185,16 @@ sub image_create_text {
 #  $font = "Helvetica" if ($font eq "");
   # create an image. We'll just set whatever size here because we want
   # to resize the image when we figure out how big the text is.
-  $image = Gimp::Image->new(64,64,&RGB); # don't waste too much  resources ;-/
-
+  $image = Gimp->image_new(64,64,RGB); # don't waste too much  resources ;-/
+    
   $tcol = Gimp->palette_get_foreground ();
   Gimp->palette_set_foreground ($fgcolor);
   # Create a layer for the text.
   $text_layer = Gimp->text($image,-1,0,0,$text,10,1,$size,
-                          &PIXELS,"*",$font,"*","*","*","*","*","*");
+                          PIXELS,"*",$font,"*","*","*","*","*","*"); 
   Gimp->palette_set_foreground ($tcol);
-
-  Gimp->layer_set_preserve_trans($text_layer, &FALSE);
+    
+  Gimp->layer_set_preserve_trans($text_layer, FALSE);
 
   # Resize the image based on size of text.
   Gimp->image_resize($image,Gimp->drawable_width($text_layer),
@@ -214,9 +222,9 @@ sub layer_add_layer_as_mask {
   my ($image,$layer,$layer_mask) = @_;
   my $mask;
 
-  Gimp->selection_all ($image);
-  $layer_mask->edit_copy ();
-  Gimp->layer_add_alpha ($layer);
+  Gimp->selection_all ($image);  
+  $layer_mask->edit_copy ();  
+  Gimp->layer_add_alpha ($layer); 
   $mask = Gimp->layer_create_mask ($layer,0);
   $mask->edit_paste (0);
   Gimp->floating_sel_anchor(Gimp->image_floating_selection($image));
@@ -228,6 +236,17 @@ sub layer_add_layer_as_mask {
 # all functions below are by Marc Lehmann
 =pod
 
+=item C<gimp_text_wh $text,$fontname>
+
+returns the width and height of the "$text" of the given font (XLFD format)
+
+=cut
+sub gimp_text_wh {
+   (Gimp->text_get_extents_fontname($_[0],xlfd_size $_[1],$_[1]))[0,1];
+}
+
+=pod
+
 =item C<gimp_image_layertype $alpha>
 
 returns the corresponding layer type for an image, alpha controls wether the layer type
@@ -249,31 +268,38 @@ alpha.
 
 mark the given layers visible (invisible) and all others invisible (visible).
 
-=cut
+=item C<gimp_layer_get_position $layer>
 
+return the position the layer has in the image layer stack.
+
+=item C<gimp_layer_set_position $layer,$new_index>
+
+moves the layer to a new position in the layer stack.
+
+=cut
 sub gimp_image_layertype {
    my($type,$alpha) = ($_[0]->base_type,$_[1]);
-   $type == &RGB     ? $alpha ? &RGBA_IMAGE     : &RGB_IMAGE     :
-   $type == &GRAY    ? $alpha ? &GRAYA_IMAGE    : &GRAY_IMAGE    :
-   $type == &INDEXED ? $alpha ? &INDEXEDA_IMAGE : &INDEXED_IMAGE :
+   $type == RGB     ? $alpha ? RGBA_IMAGE     : RGB_IMAGE     :
+   $type == GRAY    ? $alpha ? GRAYA_IMAGE    : GRAY_IMAGE    :
+   $type == INDEXED ? $alpha ? INDEXEDA_IMAGE : INDEXED_IMAGE :
    die;
 }
 
 sub gimp_layer2imagetype {
    my $type = shift;
-   $type == &RGB_IMAGE		? &RGB		:
-   $type == &RGBA_IMAGE		? &RGB		:
-   $type == &GRAY_IMAGE		? &GRAY		:
-   $type == &GRAYA_IMAGE	? &GRAY		:
-   $type == &INDEXED_IMAGE	? &INDEXED	:
-   $type == &INDEXEDA_IMAGE	? &INDEXED	:
+   $type == RGB_IMAGE		? RGB		:
+   $type == RGBA_IMAGE		? RGB		:
+   $type == GRAY_IMAGE		? GRAY		:
+   $type == GRAYA_IMAGE		? GRAY		:
+   $type == INDEXED_IMAGE	? INDEXED	:
+   $type == INDEXEDA_IMAGE	? INDEXED	:
    die;
 }
 
 sub gimp_image_add_new_layer {
    my ($image,$index,$filltype,$alpha)=@_;
-   my $layer = new Gimp::Layer ($image, $image->width, $image->height, $image->layertype (defined $alpha ? $alpha : 1), join(":",(caller)[1,2]), 100, &NORMAL_MODE);
-   $layer->fill (defined $filltype ? $filltype : &BACKGROUND_FILL);
+   my $layer = new Layer ($image, $image->width, $image->height, $image->layertype (defined $alpha ? $alpha : 1), join(":",(caller)[1,2]), 100, NORMAL_MODE);
+   $layer->fill (defined $filltype ? $filltype : BACKGROUND_FILL);
    $image->add_layer ($layer, $index*1);
    $layer;
 }
@@ -294,6 +320,30 @@ sub gimp_image_set_invisible {
    }
 }
 
+sub gimp_layer_get_position {
+   my $layer = shift;
+   my @layers = $layer->get_image->get_layers;
+   for (0..$#layers) {
+      # the my is necessary for broken perl (return $_ => undef)
+      return (my $index=$_) if ${$layers[$_]} == $$layer;
+   }
+   ();
+}
+
+sub gimp_layer_set_position {
+   my($layer,$new_pos)=@_;
+   $pos=$layer->get_position;
+   $layer->add_alpha;
+   while($pos>$new_pos) {
+      $layer->lower_layer;
+      $pos--;
+   }
+   while($pos<$new_pos) {
+      $layer->raise_layer;
+      $pos++;
+   }
+}
+
 =pod
 
 =back
